'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _createChannel = require('./create-channel');

var _createChannel2 = _interopRequireDefault(_createChannel);

var _getChannelName = require('./get-channel-name');

var _getChannelName2 = _interopRequireDefault(_getChannelName);

var _workerConsumerFactory = require('./worker-consumer-factory');

var _workerConsumerFactory2 = _interopRequireDefault(_workerConsumerFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultChannelOptions = {
  prefetch: 1
};

const defaultQueueOptions = {
  durable: true,
  maxPriority: 0
};

const defaultWorkerOptions = {
  noAck: false
};

const defaultRetryQueueOptions = {
  durable: true,
  maxPriority: 10,
  maxLength: 10000
};

const defaultDoneQueueOptions = {
  durable: true,
  maxLength: 10000
};

const addWorker = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* (args, plugin) {
    const { connection, queue: queueName, worker, options } = args;
    const { options: pluginOptions, state: pluginState } = plugin;
    const {
      retryQueue: pluginRetryOptions,
      doneQueue: pluginDoneOptions
    } = pluginOptions;

    const { _openChannels } = pluginState;

    const {
      channelName: userChannelName,
      channelOptions,
      queueOptions,
      retryQueueOptions: userRetryQueueOptions,
      retryOptions: userRetryOptions,
      doneQueueOptions: userDoneQueueOptions,
      doneOptions: userDoneOptions,
      workerOptions
    } = options || {};

    let channel;

    try {
      const channelName = userChannelName || (0, _getChannelName2.default)({ method: 'addWorker', queue: queueName });

      const rQueueOptions = (0, _defaultsDeep2.default)({}, userRetryQueueOptions, defaultRetryQueueOptions);
      const doneQueueOptions = (0, _defaultsDeep2.default)({}, userDoneQueueOptions, defaultDoneQueueOptions);

      let retryQueueOptions;
      let retryOptions;
      let doneOptions;

      if (pluginRetryOptions) {
        retryOptions = (0, _defaultsDeep2.default)({}, userRetryOptions, pluginRetryOptions);
      }

      if (pluginDoneOptions) {
        doneOptions = (0, _defaultsDeep2.default)({}, userDoneOptions, pluginDoneOptions);
      }

      if (_openChannels[channelName]) {
        channel = _openChannels[channelName].channel;
      } else {
        channel = yield (0, _createChannel2.default)({
          name: channelName,
          options: (0, _defaultsDeep2.default)({}, channelOptions, defaultChannelOptions),
          persist: true,
          connection
        }, plugin);
      }

      const queuesAsserted = [];

      // Work queue assertion
      const wQueueOptions = (0, _defaultsDeep2.default)({}, queueOptions, defaultQueueOptions);
      const workQueue = channel.assertQueue(queueName, wQueueOptions);

      queuesAsserted.push(workQueue);

      // Retry queue assertion
      if (retryOptions && retryOptions.suffix) {
        const { suffix, maxLength } = retryOptions;
        const rQueueName = `${queueName}${suffix}`;

        retryQueueOptions = (0, _defaultsDeep2.default)({}, rQueueOptions, {
          maxLength,
          deadLetterExchange: '',
          deadLetterRoutingKey: queueName
        });

        const retryQueue = channel.assertQueue(rQueueName, retryQueueOptions);
        queuesAsserted.push(retryQueue);
      } else {
        queuesAsserted.push(_promise2.default.resolve(null));
      }

      // Done queue assertion
      if (doneOptions && doneOptions.suffix) {
        const { suffix, maxLength } = doneOptions;
        const dQueueName = `${queueName}${suffix}`;

        const dQueueOptions = (0, _defaultsDeep2.default)({}, doneQueueOptions, {
          maxLength
        });

        const doneQueue = channel.assertQueue(dQueueName, dQueueOptions);
        queuesAsserted.push(doneQueue);
      } else {
        queuesAsserted.push(_promise2.default.resolve(null));
      }

      const [queue, retryQueue, doneQueue] = yield _promise2.default.all(queuesAsserted);

      const consumer = (0, _workerConsumerFactory2.default)({
        channel,
        queue,
        worker,
        retryQueue,
        retryQueueOptions,
        retryOptions,
        doneQueue,
        doneOptions
      });

      const consumerOptions = (0, _defaultsDeep2.default)({}, workerOptions, defaultWorkerOptions);

      const consumed = channel.consume(queue.queue, consumer, consumerOptions);

      return {
        channel,
        queue,
        consumed
      };
    } catch (error) {
      throw error;
    }
  });

  return function addWorker(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

exports.default = addWorker;