'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const consumerFactory = function (args) {
  const { channel, queue, subscriber, options } = args;
  const { noAck } = options || {};

  return (() => {
    var _ref = (0, _asyncToGenerator3.default)(function* (message) {
      try {
        const { content, properties } = message;
        const { contentType, contentEncoding } = properties;

        let payload = content.toString(contentEncoding);

        if (contentType === 'application/json') {
          payload = JSON.parse(payload);
        }

        const consumerObj = {
          raw: message,
          properties,
          payload,
          channel,
          queue
        };

        const resultCode = yield subscriber(consumerObj);

        // Default assumes the reply was handled manually
        if (resultCode && !noAck) {
          if (resultCode === _constants.ACK) {
            channel.ack(message);
          } else if (resultCode === _constants.NACK) {
            channel.nack(message);
          }
        }
      } catch (error) {
        if (!noAck) {
          channel.nack(message);
        }
        throw error;
      }
    });

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  })();
};

exports.default = consumerFactory;